﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System.Threading;

namespace Stepper_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
        formCleaner cleaner;
        Stepper stepper = null;
		private ErrorEventBox errorBox;

		public Form1() {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
            InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {

			//create the phidget object
			Phidget.InvokeEventCallbacks = true;
			errorBox = new ErrorEventBox(this);
			stepper = new Stepper();

			//register the various handlers for the important events on this device
			stepper.Attach += stepper_attach;
			stepper.Detach += stepper_detach;
			stepper.Error += stepper_error;
			stepper.PositionChange += stepper_positionChange;
			stepper.VelocityChange += stepper_velocityChange;
			stepper.Stopped += Stepper_Stopped;

			Phidget22.Phidget.InvokeEventCallbacks = true;

			//open the device
			open.openCmdLine(stepper);

		}

		private void updateStoppedTxt() {
			try {
				if (stepper.IsMoving) {
					stoppedTxt.Text = "Moving";
					stoppedTxt.BackColor = Color.Green;
				} else {
					stoppedTxt.Text = "Stopped";
					stoppedTxt.BackColor = Color.Red;
				}
			} catch {
				stoppedTxt.Text = "";
				stoppedTxt.BackColor = Color.Empty;
			}
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			stepper.Close();
		}

		#region phidget event handlers
		void stepper_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			Stepper stepper = (Stepper)sender;

			//set up the limits for the various controls etc...

			switch (stepper.DeviceID) {
			case DeviceID.PN_1062:
				tbCurrentLimit1.Visible = false;
				tbCurrentLimit2.Visible = false;
				minCurrLbl1.Visible = false;
				minCurrLbl2.Visible = false;
				maxCurrLbl1.Visible = false;
				maxCurrLbl2.Visible = false;
				currLimitLbl1.Visible = false;
				currLimitLbl2.Visible = false;
				txtCurrentLimit1.Visible = false;
				txtCurrentLimit2.Visible = false;
				break;

			default:
				tbCurrentLimit1.Visible = true;
				tbCurrentLimit2.Visible = true;
				minCurrLbl1.Visible = true;
				minCurrLbl2.Visible = true;
				maxCurrLbl1.Visible = true;
				maxCurrLbl2.Visible = true;
				currLimitLbl1.Visible = true;
				currLimitLbl2.Visible = true;
				txtCurrentLimit1.Visible = true;
				txtCurrentLimit2.Visible = true;
				break;
			}

			tbTargetPosition.Value = 0;

			setupMaxMin(stepper);
			updateValues(stepper);

			enBtn.Text = stepper.Engaged ? "Disengage" : "Engage";

			statusBox.Visible = true;
			outputSettings.Visible = true;
		}

		void setSliderValue(TrackBar tb, double val, double min, double max) {
			val -= min;
			val = (val * (tb.Maximum - tb.Minimum)) / (max - min);
			tb.Value = (int)Math.Round(val);
		}

		double getSliderValue(TrackBar tb, double min, double max) {
			double val = tb.Value - tb.Minimum;
			val = (val * (max - min)) / (tb.Maximum - tb.Minimum) + min;
			return val;
		}
		void setupMaxMin(Stepper stepper) {
			try {
				if (outputSettings.SelectedTab.Equals(tabPage1)) {
					minTargetPositionLbl.Text = tbTargetPosition.Minimum.ToString();
					maxTargetPositionLbl.Text = tbTargetPosition.Maximum.ToString();
                    posRangeTxt.Text = tbTargetPosition.Maximum.ToString();

					minAccelLbl1.Text = stepper.MinAcceleration.ToString();
					maxAccelLbl1.Text = stepper.MaxAcceleration.ToString();

					minVelocityLbl1.Text = stepper.MinVelocityLimit.ToString();
					maxVelocityLbl1.Text = stepper.MaxVelocityLimit.ToString();

					if (stepper.DeviceID != DeviceID.PN_1062) {
						minCurrLbl1.Text = stepper.MinCurrentLimit.ToString() + " A";
						maxCurrLbl1.Text = stepper.MaxCurrentLimit.ToString() + " A";
					}
				}

				if (outputSettings.SelectedTab.Equals(tabPage2)) {
					minAccelLbl2.Text = stepper.MinAcceleration.ToString();
					maxAccelLbl2.Text = stepper.MaxAcceleration.ToString();

					minVelocityLbl2.Text = stepper.MinVelocityLimit.ToString();
					maxVelocityLbl2.Text = stepper.MaxVelocityLimit.ToString();

					if (stepper.DeviceID != DeviceID.PN_1062) {
						minCurrLbl2.Text = stepper.MinCurrentLimit.ToString() + " A";
						maxCurrLbl2.Text = stepper.MaxCurrentLimit.ToString() + " A";
					}
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting mins / maxs: " + ex.Message);
			}
		}

		void updateValues(Stepper stepper) {

			try {
				txtPosition.Text = stepper.Position.ToString();
			} catch {
				txtPosition.Text = "N/A";
			}
			try {
				txtVelocity.Text = stepper.Velocity.ToString("f3");
			} catch {
				txtVelocity.Text = "N/A";
			}

			if (outputSettings.SelectedTab.Equals(tabPage1)) {
				try {
					setSliderValue(tbAcceleration1, stepper.Acceleration, stepper.MinAcceleration, stepper.MaxAcceleration);
					txtAcceleration1.Text = stepper.Acceleration.ToString();
				} catch {
					txtAcceleration1.Text = "N/A";
				}
				try {
					setSliderValue(tbVelocityLimit1, stepper.VelocityLimit, stepper.MinVelocityLimit, stepper.MaxVelocityLimit);
					txtVelocityLimit1.Text = stepper.VelocityLimit.ToString();
				} catch {
					txtVelocityLimit1.Text = "N/A";
				}
				try {
					tbTargetPosition.Value = (int)Math.Round(stepper.TargetPosition);
					txtTargetPosition.Text = stepper.TargetPosition.ToString();
				} catch {
					txtTargetPosition.Text = "N/A";
				}
				try {
					setSliderValue(tbCurrentLimit1, stepper.CurrentLimit, stepper.MinCurrentLimit, stepper.MaxCurrentLimit);
					txtCurrentLimit1.Text = stepper.CurrentLimit.ToString() + " A";
				} catch {
					txtCurrentLimit1.Text = "N/A";
				}
			}

			if (outputSettings.SelectedTab.Equals(tabPage2)) {
				try {
					setSliderValue(tbAcceleration2, stepper.MinAcceleration, stepper.MinAcceleration, stepper.MaxAcceleration);
					txtAcceleration2.Text = stepper.Acceleration.ToString();
				} catch {
					txtAcceleration2.Text = "N/A";
				}
				try {
					setSliderValue(tbVelocityLimit2, stepper.VelocityLimit, stepper.MinVelocityLimit, stepper.MaxVelocityLimit);
					txtVelocityLimit2.Text = stepper.VelocityLimit.ToString();
				} catch {
					txtVelocityLimit2.Text = "N/A";
				}
				try {
					setSliderValue(tbCurrentLimit2, stepper.CurrentLimit, stepper.MinCurrentLimit, stepper.MaxCurrentLimit);
					txtCurrentLimit2.Text = stepper.CurrentLimit.ToString() + " A";
				} catch {
					txtCurrentLimit2.Text = "N/A";
				}
			}
		}

		void stepper_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
            cleaner.clean();

            statusBox.Visible = false;
			outputSettings.Visible = false;
		}

		void stepper_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void stepper_positionChange(object sender, Phidget22.Events.StepperPositionChangeEventArgs e) {
			txtPosition.Text = e.Position.ToString();
			updateStoppedTxt();
		}

		void stepper_velocityChange(object sender, Phidget22.Events.StepperVelocityChangeEventArgs e) {
			txtVelocity.Text = e.Velocity.ToString("f3");
			updateStoppedTxt();
		}

		private void Stepper_Stopped(object sender, StepperStoppedEventArgs e) {

			updateStoppedTxt();
		}
		#endregion

		#region motor control handlers
		//change between run mode and position mode
		private void tabControl_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				if (outputSettings.SelectedIndex == 0) {
					stepper.ControlMode = StepperControlMode.Step;
				} else {
					stepper.ControlMode = StepperControlMode.Run;
				}
				setupMaxMin(stepper);
				updateValues(stepper);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting control mode: " + ex.Message);
			}
		}
		//set the target position in position mode
		private void targetPosBar_Scroll(object sender, EventArgs e) {
			try {
				stepper.TargetPosition = tbTargetPosition.Value;
				txtTargetPosition.Text = stepper.TargetPosition.ToString();
				updateStoppedTxt();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting target position: " + ex.Message);
			}
		}

		//adjust the amount of current the controller can deliver
		private void tbCurrentLimit_Scroll(object sender, EventArgs e) {
			try {
				stepper.CurrentLimit = getSliderValue(tbCurrentLimit1, stepper.MinCurrentLimit, stepper.MaxCurrentLimit);
				txtCurrentLimit1.Text = stepper.CurrentLimit.ToString() + " A";
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting current limit: " + ex.Message);
			}
		}
		private void tbCurrentLimit2_Scroll(object sender, EventArgs e) {
			try {
				stepper.CurrentLimit = getSliderValue(tbCurrentLimit2, stepper.MinCurrentLimit, stepper.MaxCurrentLimit);
				txtCurrentLimit2.Text = stepper.CurrentLimit.ToString() + " A";
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting current limit: " + ex.Message);
			}
		}

		//set the acceleration for when the controller is ramping up or down
		private void tbAcceleration1_Scroll(object sender, EventArgs e) {
			try {
				stepper.Acceleration = getSliderValue(tbAcceleration1, stepper.MinAcceleration, stepper.MaxAcceleration);
				txtAcceleration1.Text = stepper.Acceleration.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting acceleration: " + ex.Message);
			}
		}
		private void tbAcceleration2_Scroll(object sender, EventArgs e) {
			try {
				stepper.Acceleration = getSliderValue(tbAcceleration2, stepper.MinAcceleration, stepper.MaxAcceleration);
				txtAcceleration2.Text = stepper.Acceleration.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting acceleration: " + ex.Message);
			}
		}

		//set the velocity limit.  the controller will accelerate until it reaches this limit and then decellerate as it approaches the target position
		//in run mode the motor will be driven at this velocity constantly until stopped.
		private void tbVelocityLimit1_Scroll(object sender, EventArgs e) {
			try {
				stepper.VelocityLimit = getSliderValue(tbVelocityLimit1, stepper.MinVelocityLimit, stepper.MaxVelocityLimit);
				txtVelocityLimit1.Text = stepper.VelocityLimit.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting velocity limit: " + ex.Message);
			}
		}
		private void tbVelocityLimit2_Scroll(object sender, EventArgs e) {
			try {
				stepper.VelocityLimit = getSliderValue(tbVelocityLimit2, stepper.MinVelocityLimit, stepper.MaxVelocityLimit);
				txtVelocityLimit2.Text = stepper.VelocityLimit.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting velocity limit: " + ex.Message);
			}
		}

		//set the position of the motor to 0.  this doesn't move the motor, just resets it's internal position counter to 0 (position mode only)
		private void btnSetZeroPosition_Click(object sender, EventArgs e) {
			try {
				stepper.AddPositionOffset(-stepper.Position);
				updateValues(stepper);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting zeroing position: " + ex.Message);
			}
		}

		//stop the motor (run mode only)
		private void StopButton_Click(object sender, EventArgs e) {
			try {
				stepper.VelocityLimit = 0;
				setSliderValue(tbVelocityLimit2, stepper.VelocityLimit, stepper.MinVelocityLimit, stepper.MaxVelocityLimit);
				txtVelocityLimit2.Text = stepper.VelocityLimit.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error stopping motor: " + ex.Message);
			}
		}
		#endregion

		//engage the motor.  this will apply power to the motor and begin issuing step commands assuming a target position or target velocity has been set in either position or run mode respectively
		private void enBtn_Click(object sender, EventArgs e) {
			//toggle enable
			try {
				if (stepper.Engaged == false) {
					stepper.Engaged = true;
					enBtn.Text = "Disengage";
				} else if (stepper.Engaged == true) {
					stepper.Engaged = false;
					enBtn.Text = "Engage";
				}
			} catch (PhidgetException ex) { errorBox.addMessage("Error enabling: " + ex.ToString()); }
		}

		/* set the rescale factor.  this controls the effective units of motor position, velocity and acceleration.
         * for example, a stepper motor with a 1.8° step angle takes 200 steps to do a full rotation.
         * the phidget stepper controllers output in 1/16th step increments meaning if you set the rescale factor to
         * 200*16 a position of 1 would equate to 360°.  if you set the rescale factor to 1.8*16 a position of 1 would
         * be 1°.  this functionality just allows the user to rescale the units used in the library to something
         * more easily visualized*/
        private void rescaleSetBtn_Click(object sender, EventArgs e)
        {
            try {
                stepper.RescaleFactor = double.Parse(txtRescaleFactor.Text);
                setupMaxMin(stepper);
                updateValues(stepper);
            }
            catch(Exception ex)
            {
                errorBox.addMessage("Error setting rescale factor: " + ex.Message);
                txtRescaleFactor.Text = "1";
            }
        }

        private void posRangeSetBtn_Click(object sender, EventArgs e)
        {
            try {
                int temp = int.Parse(posRangeTxt.Text);
            
                tbTargetPosition.Maximum = temp;
                tbTargetPosition.Minimum = -temp;
                tbTargetPosition.TickFrequency = temp / 100;
                maxTargetPositionLbl.Text = temp.ToString();
                minTargetPositionLbl.Text = (-temp).ToString();
            }
            catch(Exception ex)
            { errorBox.addMessage("Error setting target range: " + ex.Message); }

        }
    }
}
